@@ -1,8 +1,7 @@
-// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
-//
+
 //  RQuantLib -- R interface to the QuantLib libraries
 //
-//  Copyright (C) 2002 - 2014  Dirk Eddelbuettel <edd@debian.org>
+//  Copyright (C) 2002 - 2018  Dirk Eddelbuettel <edd@debian.org>
 //
 //  This file is part of RQuantLib.
 //
@@ -19,7 +18,7 @@
 //  You should have received a copy of the GNU General Public License
 //  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <rquantlib.h>
+#include <rquantlib_internal.h>
 
 // the actual implementation is now in this header which is also pulled into
 // compilation units when using 'plugin' mode via Rcpp Attributes
@@ -33,4 +32,3 @@ QuantLib::Date advanceDate(QuantLib::Date issueDate, int days) {
     QuantLib::Date newDate = cal.advance(issueDate, days, QuantLib::Days);
     return(newDate);
 }
-
